/**
 *
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <string>
#include <optional>

namespace org::apache::nifi::minifi::core {

class RepositoryMetricsSource {
 public:
  struct RocksDbStats {
    uint64_t table_readers_size{};
    uint64_t all_memory_tables_size{};
  };

  virtual ~RepositoryMetricsSource() = default;
  virtual uint64_t getRepositorySize() const = 0;
  virtual uint64_t getRepositoryEntryCount() const = 0;
  virtual std::string getRepositoryName() const = 0;

  virtual uint64_t getMaxRepositorySize() const {
    return 0;
  }

  virtual bool isFull() const {
    return false;
  }

  virtual bool isRunning() const {
    return true;
  }

  virtual std::optional<RocksDbStats> getRocksDbStats() const {
    return std::nullopt;
  }
};

}  // namespace org::apache::nifi::minifi::core
